// -- Objective HUD -- v2.0.1 -------------------------------------------------------------------------------
// Written by |HH|Crunchy (Jonathan Slark)
// Email: jonslark@barrysworld.co.uk
//
// *** Requires the Presto Pack v0.93 or later ***  http://www.planetstarsiege.com/presto/
// *** Requires my ObjectiveTrak script, a new tool for scripters! v2.0 beta or later ***
// *** Requires my Events script ***
// To install, put the scripts and pictures into a directory named "Crunchy" in your "Tribes/config"
// directory.  Include the ObjectiveHUD.cs in your autoexec.cs.  Now read the notes below and edit the
// preferences to taste.
// example autoexec.cs:
//	exec("Presto\\Install.cs");
//	Include("Crunchy\\ObjectiveHUD.cs");
//
// The Objective HUD displays the names and status of each of the objectives in a mission. The HUD uses my
// ObjectiveTrak script, the script won't know about objectives until they are captured, dropped etc.
// However the script saves the information to disk for use in the future.  Play your favourite mission and
// once it knows about all the objectives it will display them all everytime you play the level.  The script
// currently works with C&H, D&D and F&R type objectives.  There is a file pack available that has
// information about all base missions.  If you download and install this then the HUD will know about the
// objectives without having to learn about them.
//
// C&H
// These objectives have a switch symbol: red means enemy controled, green means your team has control and
// black means neutral (ie start of the game).  If you join mid-game then the script won't know the status of
// the objectives, they will be yellow.
//
// D&D
// These objectives appear as generator symbols.  Red means destroyed, green means safe.  Again yellow means
// unknown.  Both the enemy and friendly objectives start out green and will turn red once destroyed.
//
// F&R
// The flags start off black: neutral.  When someone picks up a flag they will have a yellow background with
// a green or red square on it.  Green means friendly carrier, red is enemy carrier.  Grey background with
// red or green square on the flag means the enemy has the flag at their base or you do respectively.  A
// yellow sqaure on a grey background indicates unknown status or dropped flag.  Hard to visualise but it's
// obvious when you are playing :).
//
// Due to the limitation of only having 8 bitmaps in a text frame at once, if there are more than 8
// objectives then the pictures change to similar text representations.  There are very few missions with
// more than 8 objectives however.
//
//
// Changes in v2.0.1
//	+ HUD comes on automatically at start of any mission type.  The HUD is useful for most missions.
//
// What no v1.0?
//  - Version 1 worked with ObjectiveTrak v1.0 which only tracked C&H objectives.  I generalised the tracking
//	  script to track any objectives.
//
//
// -- Preferences --
//
// This places the HUD in the top right corner under the compass, edit to place it where you want to.  The
// height is the height-per-objective, the HUD resizes according to the number of objectives.  If you know
// about positioning HUDs the commented out position below is much cooler.
$CrunchyPref::ObjectiveHUDPos = "100% 20% 210 14";
//
// This placement is at the bottom right of the screen and will fit nicely with the TeamHUD.  If you use the
// TeamHUD you would need to make it 100%-210 wide and in the lower left to make it fit perfectly with the
// Objective HUD. This HUD grows in a similar way to the TeamHUD and so looks good in this position.  The HUD
// is quite wide to allow for long objective names.
//$CrunchyPref::ObjectiveHUDPos = "100% 100% 210 14";
//
// Toggle the HUD on and off with this key, uses the same style as in PrestoPrefs.
$CrunchyPref::ObjectiveHUDTogKey = "control o";
//

//
// -- Header --
//

if ($Presto::version < 0.93)
	echo("ObjectiveHUD requires Presto Pack v0.93 or later.");
else
{

Include("Presto\\Event.cs");
Include("Crunchy\\Events.cs");
Include("Presto\\HUD.cs");
Include("Presto\\TeamTrak.cs");
Include("Crunchy\\ObjectiveTrak.cs"); // This does the business, a new tool for scripters!

// Look at all the pretty events mummy :)
Event::Attach(eventClientChangeTeam, ObjectiveHUD::ResetChangeTeam);
Event::Attach(eventConnected, "ObjectiveHUD::Reset();ObjectiveHUD::Resize();");
Event::Attach(eventChangeMission, "ObjectiveHUD::Reset();ObjectiveHUD::Resize();");
Event::Attach(eventObjectiveNew, "ObjectiveHUD::Resize();HUD::Update(ObjectiveHUD);");
Event::Attach(eventObjectiveUpdated, "HUD::Update(ObjectiveHUD);");
Event::Attach(eventObjectiveLoaded, "ObjectiveHUD::Resize();HUD::Update(ObjectiveHUD);");
Event::Attach(eventObjectiveMadeNeutral, "HUD::Update(ObjectiveHUD);");

// Hand drawn and hand anti-aliased BMPs (you try and make pictures this small look cool ;)
// If ppl are using massive resolutions I may need to make these clearer?
// |HH|Rico assures me they are ok :)

// C&H
// Bitmaps for each status
$ObjectiveHUD::[$Objective::Friendly,	$Objective::CandH, bmp] = "Crunchy\\small_tower_friendly";
$ObjectiveHUD::[$Objective::Enemy,		$Objective::CandH, bmp] = "Crunchy\\small_tower_enemy";
$ObjectiveHUD::[$Objective::Unknown,	$Objective::CandH, bmp] = "Crunchy\\small_tower_unknown";
$ObjectiveHUD::[$Objective::Neutral,	$Objective::CandH, bmp] = "Crunchy\\small_tower_neutral";
// Text alternatives in case of more than 8 objectives
$ObjectiveHUD::[$Objective::Friendly,	$Objective::CandH, text] = "<f2>F";
$ObjectiveHUD::[$Objective::Enemy,		$Objective::CandH, text] = "<f0>E";
$ObjectiveHUD::[$Objective::Unknown,	$Objective::CandH, text] = "<f1>?";
$ObjectiveHUD::[$Objective::Neutral,	$Objective::CandH, text] = "<f1>N";

// D&D
$ObjectiveHUD::[$Objective::Neutral,	$Objective::DandD, bmp]	= "Crunchy\\small_objective_safe";
$ObjectiveHUD::[$Objective::Destroyed,	$Objective::DandD, bmp]	= "Crunchy\\small_objective_destroyed";
$ObjectiveHUD::[$Objective::Unknown,	$Objective::DandD, bmp]	= "Crunchy\\small_objective_unknown";
$ObjectiveHUD::[$Objective::Neutral,	$Objective::DandD, text] = "<f2>S";
$ObjectiveHUD::[$Objective::Destroyed,	$Objective::DandD, text] = "<f0>D";
$ObjectiveHUD::[$Objective::Unknown,	$Objective::DandD, text] = "<f1>?";

// CTF - not implemented yet
//$ObjectiveHUD::[$Objective::Neutral,	$Objective::CTF, bmp] = "Crunchy\\small_flag_friendly";
//$ObjectiveHUD::[$Objective::Enemy,		$Objective::CTF, bmp] = "Crunchy\\small_flag_enemy";
//$ObjectiveHUD::[$Objective::Dropped,	$Objective::CTF, bmp] = "Crunchy\\small_flag_dropped";
//$ObjectiveHUD::[$Objective::Unknown,	$Objective::CTF, bmp] = "Crunchy\\small_flag_unknown";

// F&R
$ObjectiveHUD::[$Objective::Friendly,		$Objective::FandR, bmp] = "Crunchy\\small_flag_friendly";
$ObjectiveHUD::[$Objective::Enemy,			$Objective::FandR, bmp] = "Crunchy\\small_flag_enemy";
$ObjectiveHUD::[$Objective::FriendlyCarry,	$Objective::FandR, bmp] = "Crunchy\\small_flag_friendlyCarry";
$ObjectiveHUD::[$Objective::EnemyCarry,		$Objective::FandR, bmp] = "Crunchy\\small_flag_enemyCarry";
$ObjectiveHUD::[$Objective::Dropped,		$Objective::FandR, bmp] = "Crunchy\\small_flag_dropped";
$ObjectiveHUD::[$Objective::Unknown,		$Objective::FandR, bmp] = "Crunchy\\small_flag_unknown";
$ObjectiveHUD::[$Objective::Neutral,		$Objective::FandR, bmp] = "Crunchy\\small_flag_neutral";
$ObjectiveHUD::[$Objective::Friendly,		$Objective::FandR, text] = "<f2>F";
$ObjectiveHUD::[$Objective::Enemy,			$Objective::FandR, text] = "<f0>E";
$ObjectiveHUD::[$Objective::FriendlyCarry,	$Objective::FandR, text] = "<f2>C";
$ObjectiveHUD::[$Objective::EnemyCarry,		$Objective::FandR, text] = "<f0>C";
$ObjectiveHUD::[$Objective::Dropped,		$Objective::FandR, text] = "<f0>D";
$ObjectiveHUD::[$Objective::Unknown,		$Objective::FandR, text] = "<f1>?";
$ObjectiveHUD::[$Objective::Neutral,		$Objective::FandR, text] = "<f1>N";

Presto::AddScriptBanner(ObjectiveHUD,
	" <f2>Objective HUD <jr><f0>version 2.0.1 <jl>\n" @
	" \n" @
	" <f0>Displays mission objectives.\n" @
	" <f0>Uses <f2>ObjectiveTrak<f0>, a new\n" @
	" <f0>tool for scripters!\n" @
	" <f1> Toggle <f0>is on <f1>"@$CrunchyPref::ObjectiveHUDTogKey@"<f0>.\n" @
	" \n" @
	" <f0>Written by: <f1>|HH|Crunchy\n" @
	" <f1>jonslark@barrysworld.co.uk");

//
// -- Begin code --
//

// Update the HUD using ObjectiveTrak.
function ObjectiveHUD::UpdateHUD()
{
	// Get around 8 bitmap limit by using text if more than 8 objectives.
	%num = Objective::Number();
	if(%num > 8)
		%style = text;
	else
		%style = bmp;

	if(%num == 0)
		HUD::AddTextLine(ObjectiveHUD, "<jl><f1>No known objectives.");
	else
	{
		%pos = "0,4";
		for(%i = 0; %i < %num; %i++)
		{
			%status = Objective::GetStatus(%i);
			%client = Objective::GetClient(%i);
			%type = Objective::GetType(%i);
			%bmp = $ObjectiveHUD::[%status, %type, %style];

			%o = Objective::GetNameStripped(%i);
			if(%type == $Objective::DandD)
				%o = Objective::InsertTeamName(%o);

			if(%style == bmp)
				HUD::AddTextLine(ObjectiveHUD, "<jl><f1><B"@%pos@":"@%bmp@".bmp><L4>"@%o);
			else
				HUD::AddTextLine(ObjectiveHUD, "<jl>"@%bmp@"<f1> : "@%o);
			%pos = "-20,4";  // Sucky bitmap positioning :(
		}
	}

	return 0;  // Only update HUD when objectives are updated.
}

// If we find out about new objectives resize the HUD accordingly.
function ObjectiveHUD::Resize()
{
	%numLines = Objective::Number();
	if(%numLines < 1) %numlines = 1;
	%height = %numLines * getWord($CrunchyPref::ObjectiveHUDPos, 3);
	HUD::SetCoord(ObjectiveHUD, height, %height);
}

// Update HUD.
function ObjectiveHUD::Reset()
{
	HUD::Update(ObjectiveHUD);
}

// If we change teams HUD needs updating.
function ObjectiveHUD::ResetChangeTeam(%client, %team)
{
	if(%client != getManagerID()) return;

	HUD::Update(ObjectiveHUD);
}

// Either create the HUD or move it exists, good for testing.
if(HUD::Exists(ObjectiveHUD))
{
	ObjectiveHUD::Resize();
	HUD::Update(ObjectiveHUD);
}
else
{
	HUD::New(ObjectiveHUD, ObjectiveHUD::UpdateHUD, $CrunchyPref::ObjectiveHUDPos);
	HUD::Display(ObjectiveHUD, true);
}

// Uses Presto's cool key binding function :)
bindKey(play, $CrunchyPref::ObjectiveHUDTogKey,"HUD::ToggleDisplay(ObjectiveHUD);");


} // Presto Pack check.
